@@ -357,6 +357,18 @@ finbif_occurrence_load <- function(
 
     short_nms <- short_nms[df_names]
 
+    which_missing <- which(na_nms)
+
+    missing_short_names <- gsub("\\.", "", df_names[which_missing])
+
+    missing_short_names <- abbreviate(
+      missing_short_names, 10, FALSE, strict = TRUE, method = "both.sides"
+    )
+
+    short_nms[which_missing] <- missing_short_names
+
+    names(short_nms)[which_missing] <- df_names[which_missing]
+
     short_fcts <- grep("_fact__", df_names, value = TRUE)
 
     short_fcts <- sub("^.*_fact__", "", short_fcts)
@@ -375,14 +387,6 @@ finbif_occurrence_load <- function(
 
     short_nms[is.na(short_nms)] <- paste0("f", short_fact_sq, short_fcts)
 
-    missing <- which(short_nms == "f")
-
-    nms_missing <- df_names[missing]
-
-    nms_missing <- gsub("[^A-Za-z]", "", nms_missing)
-
-    short_nms[missing] <- abbreviate(nms_missing, 10L)
-
     select[["user"]] <- short_nms
 
     names(fb_occurrence_df) <- short_nms
@@ -1029,7 +1033,9 @@ dt_read <- function(fb_occurrence_obj) {
 
   args[["header"]] <- FALSE
 
-  df <- structure(vector("list", length(args[["select"]])), class = "data.frame")
+  df <- structure(
+    vector("list", length(args[["select"]])), class = "data.frame"
+  )
 
   if (file.exists(args[["input"]])) {
 
